from ..models import MessageListResult
from ..models import Message
from ..models import MailosaurException

class MessagesOperations(object):
    """MessagesOperations operations.
    """

    def __init__(self, session, base_url):
        self.session = session
        self.base_url = base_url

    def get(self, id):
        """Retrieve a message.

        Retrieves the detail for a single email message. Simply supply the
        unique identifier for the required message.

        :param id: The identifier of the email message to be retrieved.
        :type id: str        
        :return: Message
        :rtype: ~mailosaur.models.Message
        :raises:
         :class:`MailosaurException<mailosaur.models.MailosaurException>`
        """
        url = "%sapi/messages/%s" % (self.base_url, id)
        response = self.session.get(url)
        
        if response.status_code not in [200]:
            raise MailosaurException(response)

        data = response.json()

        return Message(data)

    def delete(self, id):
        """Delete a message.

        Permanently deletes a message. This operation cannot be undone. Also
        deletes any attachments related to the message.

        :param id: The identifier of the message to be deleted.
        :type id: str        
        :return: None
        :rtype: None
        :raises:
         :class:`MailosaurException<mailosaur.models.MailosaurException>`
        """
        url = "%sapi/messages/%s" % (self.base_url, id)
        response = self.session.delete(url)
        
        if response.status_code not in [204]:
            raise MailosaurException(response)

    def list(self, server, page=None, items_per_page=None):
        """List all messages.

        Returns a list of your messages in summary form. The summaries are
        returned sorted by received date, with the most recently-received
        messages appearing first.

        :param server: The identifier of the server hosting the messages.
        :type server: str
        :param page: Used in conjunction with `itemsPerPage` to support
         pagination.
        :type page: int
        :param items_per_page: A limit on the number of results to be returned
         per page. Can be set between 1 and 1000 items, the default is 50.
        :type items_per_page: int        
        :return: MessageListResult
        :rtype: ~mailosaur.models.MessageListResult
        :raises:
         :class:`MailosaurException<mailosaur.models.MailosaurException>`
        """
        url = "%sapi/messages" % (self.base_url)
        params = {'server': server, 'page': page, 'itemsPerPage': items_per_page}
        response = self.session.get(url, params=params)
        
        if response.status_code not in [200]:
            raise MailosaurException(response)

        data = response.json()

        return MessageListResult(data)

    def delete_all(self, server):
        """Delete all messages.

        Permanently deletes all messages held by the specified server. This
        operation cannot be undone. Also deletes any attachments related to
        each message.

        :param server: The identifier of the server to be emptied.
        :type server: str        
        :return: None
        :rtype: None
        :raises:
         :class:`MailosaurException<mailosaur.models.MailosaurException>`
        """
        url = "%sapi/messages" % (self.base_url)
        params = {'server': server}
        response = self.session.delete(url, params=params)
        
        if response.status_code not in [204]:
            raise MailosaurException(response)

    def search(self, server, criteria, page=None, items_per_page=None):
        """Search for messages.

        Returns a list of messages matching the specified search criteria, in
        summary form. The messages are returned sorted by received date, with
        the most recently-received messages appearing first.

        :param server: The identifier of the server hosting the messages.
        :type server: str
        :param criteria: The search criteria to match results against.
        :type criteria: ~mailosaur.models.SearchCriteria
        :param page: Used in conjunction with `itemsPerPage` to support
         pagination.
        :type page: int
        :param items_per_page: A limit on the number of results to be returned
         per page. Can be set between 1 and 1000 items, the default is 50.
        :type items_per_page: int        
        :return: MessageListResult
        :rtype: ~mailosaur.models.MessageListResult
        :raises:
         :class:`MailosaurException<mailosaur.models.MailosaurException>`
        """
        url = "%sapi/messages/search" % (self.base_url)
        params = {'server': server, 'page': page, 'itemsPerPage': items_per_page}
        response = self.session.post(url, params=params, json=criteria.toJSON())

        if response.status_code not in [200]:
            raise MailosaurException(response)

        data = response.json()

        return MessageListResult(data)

    def wait_for(self, server, criteria):
        """Wait for a specific message.

        Returns as soon as a message matching the specified search criteria is
        found. This is the most efficient method of looking up a message.

        :param server: The identifier of the server hosting the message.
        :type server: str
        :param criteria: The search criteria to use in order to find a match.
        :type criteria: ~mailosaur.models.SearchCriteria        
        :return: Message
        :rtype: ~mailosaur.models.Message
        :raises:
         :class:`MailosaurException<mailosaur.models.MailosaurException>`
        """
        url = "%sapi/messages/await" % (self.base_url)
        params = {'server': server}
        response = self.session.post(url, params=params, json=criteria.__dict__)
        
        if response.status_code not in [200]:
            raise MailosaurException(response)
            
        data = response.json()

        return Message(data)


# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from .spam_assassin_rule import SpamAssassinRule
from .spam_filter_results import SpamFilterResults
from .spam_analysis_result import SpamAnalysisResult
from .mailosaur_error import MailosaurError, MailosaurException
from .message_address import MessageAddress
from .link import Link
from .image import Image
from .message_content import MessageContent
from .attachment import Attachment
from .message_header import MessageHeader
from .metadata import Metadata
from .message import Message
from .message_summary import MessageSummary
from .message_list_result import MessageListResult
from .search_criteria import SearchCriteria
from .forwarding_rule import ForwardingRule
from .server import Server
from .server_list_result import ServerListResult
from .server_create_options import ServerCreateOptions

__all__ = [
    'SpamAssassinRule',
    'SpamFilterResults',
    'SpamAnalysisResult',
    'MailosaurError', 'MailosaurException',
    'MessageAddress',
    'Link',
    'Image',
    'MessageContent',
    'Attachment',
    'MessageHeader',
    'Metadata',
    'Message',
    'MessageSummary',
    'MessageListResult',
    'SearchCriteria',
    'ForwardingRule',
    'Server',
    'ServerListResult',
    'ServerCreateOptions',
]



# coding=utf-8
import dateutil.parser
from .message_address import MessageAddress
from .message_content import MessageContent
from .attachment import Attachment
from .metadata import Metadata

class Message(object):
    """Message.

    :param id: Unique identifier for the message.
    :type id: str
    :param sender: The sender of the message.
    :type sender: list[~mailosaur.models.MessageAddress]
    :param to: The message’s recipient.
    :type to: list[~mailosaur.models.MessageAddress]
    :param cc: Carbon-copied recipients for email messages.
    :type cc: list[~mailosaur.models.MessageAddress]
    :param bcc: Blind carbon-copied recipients for email messages.
    :type bcc: list[~mailosaur.models.MessageAddress]
    :param received: The datetime that this message was received by Mailosaur.
    :type received: datetime
    :param subject: The message’s subject.
    :type subject: str
    :param html: Message content that was sent in HTML format.
    :type html: ~mailosaur.models.MessageContent
    :param text: Message content that was sent in plain text format.
    :type text: ~mailosaur.models.MessageContent
    :param attachments: An array of attachment metadata for any attached
     files.
    :type attachments: list[~mailosaur.models.Attachment]
    :param metadata:
    :type metadata: ~mailosaur.models.Metadata
    :param server: Identifier for the server in which the message is located.
    :type server: str
    """

    def __init__(self, data=None):
        if data is None:
            data = {}

        self.id = data.get('id', None)
        self.sender = [MessageAddress(i) for i in data.get('from', [])]
        self.to = [MessageAddress(i) for i in data.get('to', [])]
        self.cc = [MessageAddress(i) for i in data.get('cc', [])]
        self.bcc = [MessageAddress(i) for i in data.get('bcc', [])]
        self.received = dateutil.parser.parse(data.get('received', None))
        self.subject = data.get('subject', None)
        self.html = MessageContent(data.get('html', None))
        self.text = MessageContent(data.get('text', None))
        self.attachments = [Attachment(i) for i in data.get('attachments', [])]
        self.metadata = Metadata(data.get('metadata', None))
        self.server = data.get('server', None)



from .message_summary import MessageSummary

class MessageListResult(object):
    """The result of a message listing request.

    :param items: The individual summaries of each message forming the result.
     Summaries are returned sorted by received date, with the most
     recently-received messages appearing first.
    :type items: list[~mailosaur.models.MessageSummary]
    """

    def __init__(self, data=None):
        if data is None:
            data = {}

        self.items = [MessageSummary(i) for i in data.get('items', None)]



class Attachment(object):
    """Attachment.

    :param id:
    :type id: str
    :param content_type:
    :type content_type: str
    :param file_name:
    :type file_name: str
    :param content_id:
    :type content_id: str
    :param length:
    :type length: int
    :param url:
    :type url: str
    """

    def __init__(self, data=None):
        if data is None:
            data = {}

        self.id = data.get('id', None)
        self.content_type = data.get('contentType', None)
        self.file_name = data.get('fileName', None)
        self.content_id = data.get('contentId', None)
        self.length = data.get('length', None)
        self.url = data.get('url', None)


from ..models import MailosaurException

class FilesOperations(object):
    """FilesOperations operations.
    """

    def __init__(self, session, base_url):
        self.session = session
        self.base_url = base_url

    def get_attachment(self, id):
        """Download an attachment.

        Downloads a single attachment. Simply supply the unique identifier for
        the required attachment.

        :param id: The identifier of the attachment to be downloaded.
        :type id: str
        :return: object
        :rtype: Generator
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        url = "%sapi/files/attachments/%s" % (self.base_url, id)
        response = self.session.get(url, stream=True)
        
        if response.status_code not in [200]:
            raise MailosaurException(response)

        return response

    def get_email(self, id):
        """Download EML.

        Downloads an EML file representing the specified email. Simply supply
        the unique identifier for the required email.

        :param id: The identifier of the email to be downloaded.
        :type id: str
        :return: object
        :rtype: Generator
        :raises:
         :class:`HttpOperationError<msrest.exceptions.HttpOperationError>`
        """
        url = "%sapi/files/email/%s" % (self.base_url, id)
        response = self.session.get(url, stream=True)
        
        if response.status_code not in [200]:
            raise MailosaurException(response)

        return response


from .forwarding_rule import ForwardingRule

class Server(object):
    """Server.

    :param id: Unique identifier for the server. Used as username for
     SMTP/POP3 authentication.
    :type id: str
    :param password: SMTP/POP3 password.
    :type password: str
    :param name: A name used to identify the server.
    :type name: str
    :param users: Users (excluding administrators) who have access to the
     server.
    :type users: list[str]
    :param messages: The number of messages currently in the server.
    :type messages: int
    :param forwarding_rules: The rules used to manage email forwarding for
     this server.
    :type forwarding_rules: list[~mailosaur.models.ForwardingRule]
    """

    def __init__(self, data=None):
        if data is None:
            data = {}

        self.id = data.get('id', None)
        self.password = data.get('password', None)
        self.name = data.get('name', None)
        self.users = data.get('users', None)
        self.messages = data.get('messages', None)
        self.forwarding_rules = [ForwardingRule(i) for i in data.get('forwardingRules', [])]



from ..models import SpamAnalysisResult
from ..models import MailosaurException

class AnalysisOperations(object):
    """AnalysisOperations operations.
    """

    def __init__(self, session, base_url):
        self.session = session
        self.base_url = base_url

    def spam(self, email):
        """Perform a spam test.

        Perform spam testing on the specified email.

        :param email: The identifier of the email to be analyzed.
        :type email: str        
        :return: SpamAnalysisResult
        :rtype: ~mailosaur.models.SpamAnalysisResult
        :raises:
         :class:`MailosaurException<mailosaur.models.MailosaurException>`
        """
        url = "%sapi/analysis/spam/%s" % (self.base_url, email)
        response = self.session.get(url)
        
        if response.status_code not in [200]:
            raise MailosaurException(response)

        data = response.json()

        return SpamAnalysisResult(data)



"""
    mailosaur.com API library. Basic usage:

    >>> from mailosaur import Mailosaur
    >>> mailbox = Mailosaur("BOX_ID", "YOUR_API_KEY")
    >>> emails = mailbox.get_emails()

    More options at https://mailosaur.com/docs/email/
"""

import uuid
import requests

from .operations.servers_operations import ServersOperations
from .operations.messages_operations import MessagesOperations
from .operations.analysis_operations import AnalysisOperations
from .operations.files_operations import FilesOperations

class MailosaurClient(object):
    """ Main class to access Mailosaur.com api. """

    def __init__(self, api_key, base_url="https://mailosaur.com/"):
        """ Pass in your mailbox id and api key to authenticate """
        session = requests.Session()
        session.auth = (api_key, '')
        session.headers.update({'User-Agent': 'mailosaur-python/5.0.5'})

        self.servers = ServersOperations(session, base_url)
        self.messages = MessagesOperations(session, base_url)
        self.analysis = AnalysisOperations(session, base_url)
        self.files = FilesOperations(session, base_url)



class MessageHeader(object):
    """MessageHeader.

    :param field: Header key.
    :type field: str
    :param value: Header value.
    :type value: str
    """

    def __init__(self, data=None):
        if data is None:
            data = {}

        self.field = data.get('field', None)
        self.value = data.get('value', None)
